% Represent the samples with the sparse coefficients. SIFT feature is used
% here
% input:
%     X: the warp images set X
%     tmpl: the classifier parameters
%     opt: golbal parameters
% output: 
%     A: the learned coefficient set

function A = SparseRepresentIm(X, opt, tmpl)


% the size of image patch is fixed for each particle
im_h = opt.tmplsize(1);
im_w = opt.tmplsize(2);

gridSpacing = opt.gridSpacing;
patchSize = opt.patchSize;
nrml_threshold = opt.nrml_threshold;

% make grid sampling SIFT descriptors
remX = mod(im_w-patchSize,gridSpacing);
offsetX = floor(remX/2)+1;
remY = mod(im_h-patchSize,gridSpacing);
offsetY = floor(remY/2)+1;

[gridX,gridY] = meshgrid(offsetX:gridSpacing:im_w-patchSize+1, offsetY:gridSpacing:im_h-patchSize+1);
feaSet.x = gridX(:) + patchSize/2 - 0.5;
feaSet.y = gridY(:) + patchSize/2 - 0.5;
feaSet.width = im_w;
feaSet.height = im_h;

A = [];
num_samples = size(X,3);
num_sift = size(feaSet.x,1);

% change to patch type
%=====================================================================
% first step - extract SIFT feature for all samples
% record the SIFT features from all particles and arrange them in a matrix.
SIFT_fea = zeros(128, num_samples*num_sift);
dSize = size(tmpl.D, 2);  

if opt.ispool == 1
    % for pooling
    tmp = opt.pyramid;
    tmp = sum(tmp.^2);
    A = zeros(dSize*tmp, num_samples);
else
    % for concatenation
    A = zeros(dSize*num_sift, num_samples); % for concatenation
end


for i=1:num_samples % the number of positive and negtive samples  
    tmp = X(:,:,i);
    I = tmp;
    % find SIFT descriptors
    siftArr = sp_find_sift_grid_new(I, gridX, gridY, patchSize, 0.8);

    [siftArr, siftlen] = sp_normalize_sift(siftArr, nrml_threshold); 
    SIFT_fea(:,1+(i-1)*num_sift:i*num_sift) = siftArr';
end


%% using the code wroten by ourselves
% sparse coding with the SIFT matrix for all particles
d_b = -tmpl.D' * SIFT_fea; 
% B = D' * D + opt.lambda*eye(size(D,2));
d_B = tmpl.B;
d_x0 = zeros(size(tmpl.D,2),size(SIFT_fea, 2));
% tic;
d_alpha = ones(1,size(SIFT_fea,2));
d_idx = d_alpha;
d_rho = 0.5;
d_c = 1e-6;
d_lsiter = 5;
d_maxiter = 10;
all_codes = ls_Coordinate_Descent(d_B,d_b,d_x0,d_alpha,d_idx,...
    0.15,d_rho,d_c,d_lsiter,d_maxiter);

%% 
for i=1:num_samples % the number of positive and negtive samples    
    tmp = all_codes(:,1+(i-1)*num_sift:i*num_sift);
    if opt.ispool == 1    % pooling        
        beta = Pooling(feaSet, dSize, tmp, opt.pyramid);
        A(:,i) = beta;
    else     % no pooling, just concatenating        
        A(:,i) = tmp(:);
    end
end
