% Represent the samples with the sparse coefficients. SIFT feature is used
% here
% input:
%     X: the warp images set X
%     tmpl: the classifier parameters
%     opt: golbal parameters
% output: 
%     A: the learned coefficient set

function A = SparseRepresentImGPU(X, opt, D)

% the size of image patch is fixed for each particle
im_h = opt.tmplsize(1);
im_w = opt.tmplsize(2);

gridSpacing = opt.gridSpacing;
patchSize = opt.patchSize;
nrml_threshold = opt.nrml_threshold;

% make grid sampling SIFT descriptors
remX = mod(im_w-patchSize,gridSpacing);
offsetX = floor(remX/2)+1;
remY = mod(im_h-patchSize,gridSpacing);
offsetY = floor(remY/2)+1;

[gridX,gridY] = meshgrid(offsetX:gridSpacing:im_w-patchSize+1, offsetY:gridSpacing:im_h-patchSize+1);


% num_samples = size(X,3);
siftLens = [];
A = [];
num_samples = size(X,3);

% change to patch type
%=====================================================================
% first step - extract SIFT feature fro all samples
% record the SIFT features from all particles and arrange them in a matrix.
SIFT_fea = [];
SIFT_x = [];
SIFT_y = [];
for i=1:num_samples % the number of positive and negtive samples  
    tmp = X(:,:,i);
    I = tmp;
    % find SIFT descriptors
    siftArr = sp_find_sift_grid(I, gridX, gridY, patchSize, 0.8);
    [siftArr, siftlen] = sp_normalize_sift(siftArr, nrml_threshold); 
    feaSet.feaArr = siftArr';
    SIFT_fea = [SIFT_fea feaSet.feaArr];    
    feaSet.x = gridX(:) + patchSize/2 - 0.5;
    feaSet.y = gridY(:) + patchSize/2 - 0.5;
    SIFT_x = [SIFT_x feaSet.x];
    SIFT_y = [SIFT_y feaSet.y];  
end
%======================================================================
% sparse coding with the SIFT matrix for all particles
b = -D' * SIFT_fea; 
B = D' * D + opt.lambda*eye(size(D,2));
%------------------------------------------------

d_x0 = zeros(size(D,2),size(SIFT_fea, 2),GPUsingle);
d_b = GPUsingle(b);
d_B = GPUsingle(B);
% tic;

d_alpha = ones(1,size(SIFT_fea,2),GPUsingle);
d_idx = d_alpha;
d_rho = 0.5;
d_c = 1e-6;
d_lsiter = 5;
d_maxiter = 50;
% tic;
tmp = gpu_ls_Coordinate_Descent(d_B,d_b,d_x0,d_alpha,d_idx,...
    0.15,d_rho,d_c,d_lsiter,d_maxiter);
all_codes = single(tmp);
% toc;

%================================================================

% pooling
% tic;
dSize = size(D, 2);  
for i=1:num_samples % the number of positive and negtive samples    
    feaSet.feaArr = SIFT_fea(:,1+(i-1)*9:i*9);
    feaSet.x = SIFT_x(:,i);                    
    feaSet.y = SIFT_y(:,i);
    feaSet.width = im_w;
    feaSet.height = im_h;
    beta = Pooling(feaSet, dSize, all_codes(:,1+(i-1)*9:i*9), opt.pyramid);   
    A = [A beta]; %0.1s/sample. Two slow. The speed is mainly dependent on the coding process. 10.13s   
end


