%% Sparse tracking with the learned sift-based dictionary
% try the Collaborative representation
%% Copyright (C)Qing Wang.
%% All rights reserved.
%% parameter setting
clear all;
close all;
clear;
clc;
addpath(genpath('fast_sc\'));
addpath(genpath('liblinear-1.51\'));
iptsetpref('ImshowBorder', 'tight'); %drop the display boundary

% load the offline Dictionary
load('SDic.mat','D'); % Dictionary for SIFT
tmpl.D = D;

%dir
prefix = 'Sylvester\';

D = dir(prefix);
fNum = size(D,1)-4; %

%location of the tracking result
prefix_result = 'result\';
% start frame
fstart = 1;  
% first frame
fname = sprintf('%s%04d.jpeg',prefix,fstart);  %


%read the sequence
im = imread(fname);
f_frame = sprintf('%d',1);
fprintf('tracking frame: the %sth frame\n', f_frame); 

figure(1);
imshow(im);


%% Select the target object manually
% %get 4 vertices of object
% fprintf('Please click 4 corners of the object to track:');
% [xbox,ybox] = ginput(4);  %user inputs corners of bounding box
% %get object bounding box
% xbox = round(xbox);   %get integer value
% ybox = round(ybox);
% left = min(xbox);    
% right = max(xbox); 
% top = min(ybox);     
% bottom = max(ybox);
% % objBox = [left,right,top,bottom];      
% objCenterx = round((left+right)/2);
% objCentery = round((top+bottom)/2);
% width = right-left;
% height = bottom-top;
% %the intialized parameters
% p = [objCenterx, objCentery, width, height, 0];  %x, y, w, h, rotation, angle

%% Select the target object automatically 
p = [145 81 53 53 -0.2]; %sylwester 


%% The parameters of particle filtering
opt = struct('numsample', 100, 'condenssig',1, 'posNum',30, 'negNum',60, ...
    'tmplsize', [32, 32], 'batchsize',5,'r',2, 'gridSpacing',8, ...
    'patchSize',16, 'maxImSize',32, 'nrml_threshold',1, 'gamma', 0.15, ...
    'lambda', 0.01, 'ispool', 1, 'pyramid', [1, 2, 3], 'conf', 0.5, 'ff', 0.8); %'pyramid', [1, 2, 3]
opt.posSig = [2,2,.01,.02,.002,.001];

tmpl.B = tmpl.D' * tmpl.D + opt.lambda*eye(size(tmpl.D,2));


%%
opt.affsig = [4,4,.01,.02,.002,.001]; %sylvester

%x, y, width (normalized), rotation angle, height, skew 
param0 = [p(1), p(2), p(3)/opt.tmplsize(2), p(5), p(4)/p(3), 0];
param0 = affparam2mat(param0);
param.est =  param0';

frame = rgb2gray(im);
frame =  double(frame)/256;

% draw initial track window
figure(1);
%function
drawbox(opt.tmplsize(2),opt.tmplsize(2), param.est, 'Color','r', 'LineWidth',2.5);
%record the tracking result in the first frame
Current_title = sprintf('# %d',fstart);
text(5, 18, Current_title, 'Color','y', 'FontWeight','bold', 'FontSize',12);
wname = sprintf('%s%04d.jpeg', prefix_result, fstart); % 
imwrite(frame2im(getframe(gcf)),wname);
step = 1;
hwin=gcf;
frameIndex = fstart;
Current_title = sprintf('The %dth frame',frameIndex);
set(hwin, 'Name', Current_title);

paramRes = zeros(6,fNum);
paramRes(:,fstart) = param.est; 

%% Initialize the claffifier in the first tracking frame
% select a lot of positive samples
[posSample, posPosition] = SelectPos(frame, opt.tmplsize, param.est, opt.posNum, opt.posSig);
% select a lot of negetive samples
[negSample, position] = SelectNeg(frame, param.est, opt);

X = cat(3, posSample, negSample);
% ================================================================
% Extract the descriptor based on SIFT, and coding them by the learned dictionary
F = SparseRepresentIm(X, opt, tmpl);       %SIFT
% F = SparseRepresentImGPU(X, opt, tmpl.D);  %SIFT+GPU
% ================================================================


%% Train the logistic classifier
% X data
% L label
L1 = ones(1,opt.posNum);
L2 = -ones(1,opt.negNum);
L = [L1 L2];
dim_c = size(F,1);
% tic;
model = train( L', sparse(F), '-s 0 -B 1 -c 10', 'col' );  %the character after-c is the regularization parameter for linear SVM; increase the regularization param
w = model.w(1:dim_c); 
w = w';
b = model.w(dim_c+1); 
% toc;

%store the initial model
tmpl.w = w;
tmpl.b = b;
% tmpl.D = D;
save('initModel.mat', 'tmpl');
% define an online detector
tmpl.wo = w;
tmpl.bo = b;

% ========================================================================
% test the training data to compare the classification results and the
% ground truth for the positive samples
F1 = F(:,1:opt.posNum);
P1 = 1 ./ (1 + exp(-(w'*F1+b)));
% for the negtive samples:
F0 = F(:,opt.posNum+1:opt.posNum+opt.negNum);
P0 = 1 ./ (1 + exp(-(w'*F0+b)));


% ========================================================================

paramConf = zeros(1, fNum);
paramConf(1) = P1(opt.posNum);

%% for classifier update
rho = 1e-2;
nu = 1e-5;
timeRec = zeros(1,fNum);
IsUpdate = zeros(1,fNum);
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                             %The main tracking algorithm
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
obserSet = [];

%tracking from the 2th frame
for frameIndex = fstart+1:step:fNum
    tic;
    % read the next frame
    fname = sprintf('%s%04d.jpeg',prefix,frameIndex);  %
 
    im = imread(fname);
    %display
    figure(1);
    imshow(im);   
    f_frame = sprintf('%d',frameIndex);
    fprintf('tracking frame: the %sth frame\n', f_frame);       
    frame = rgb2gray(im); 
    frame = double(frame)/256;
    
    % Using particle filtering
    last_est = param.est;
    param = estwarp_condens(frame, tmpl, param, opt);    
    
    % record the tracking result
    paramRes(:,frameIndex) = param.est;
    paramConf(frameIndex) = param.conf;
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%Update the classifier%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % update every 10 frames    
    % Store temporary positive samples
    obserSet = [obserSet param.obser];
    rho = 0.2;  
     if (size(obserSet,2) == opt.batchsize)
         % Collect some negative samples around the positive one
         [negSample, position] = SelectNeg(frame, param.est, opt);
         % Calculate the sparse representation of the negtive samples
%          tmpNeg = SparseRepresentImGPU(negSample, opt, tmpl.D);
         tmpNeg = SparseRepresentIm(negSample, opt, tmpl);
         train_data = [obserSet tmpNeg];
         % label
         L_pos = ones(1,opt.batchsize);
         L_neg = -ones(1,opt.negNum);
         L = [L_pos L_neg]; 
         
         % directly learn a new SVM classifier
         model = train( L', sparse(train_data), '-s 0 -B 1 -c 10', 'col' );  %the character after-c is the regularization parameter for linear SVM; increase the regularization param
         w = model.w(1:dim_c); w = w';
         b = model.w(dim_c+1); 
         %b = b;
         tmpl.wo  = w;
         tmpl.bo = b;
         obserSet = [];
     end
    
 
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%Display the tracking result%%%%%%%%%%%%%%%%%%%%%%%%
    %% draw result    
    figure(1);
    drawbox(opt.tmplsize(2),opt.tmplsize(2), param.est, 'Color','r', 'LineWidth',2.5);
    Current_title = sprintf('# %d',frameIndex);
    set(hwin, 'Name', Current_title);
    text(5, 18, Current_title, 'Color','y', 'FontWeight','bold', 'FontSize',12);    
    wname = sprintf('%s%04d.jpeg', prefix_result, frameIndex); %
    imwrite(frame2im(getframe(gcf)),wname);    
    timeRec(frameIndex) = toc;
end
paraSaveresult = sprintf('%s%s.mat',prefix_result,'result'); 
save(paraSaveresult, 'paramRes', 'paramConf', 'opt', 'IsUpdate', 'timeRec');



